import os
import warnings

import librosa
import torch
from sziszapangma.integration.service_core.asr.asr_base_processor import AsrBaseProcessor
from sziszapangma.integration.service_core.asr.asr_result import AsrResult
from transformers import Wav2Vec2ForCTC, Wav2Vec2Processor

warnings.filterwarnings("ignore")


# MODEL_ID = "jonatasgrosman/wav2vec2-large-xlsr-53-polish"
# SAMPLING_RATE = 16_000


class SpeechbrainAsrProcessor(AsrBaseProcessor):
    _model_id: str
    _processor: Wav2Vec2Processor
    _model: Wav2Vec2ForCTC
    _sampling_rate: int

    def __init__(self, model_id: str, sampling_rate: int):
        super().__init__()
        self._model_id = model_id
        self._sampling_rate = sampling_rate
        self._processor = Wav2Vec2Processor.from_pretrained(model_id, cache_dir='/models')
        self._model = Wav2Vec2ForCTC.from_pretrained(model_id, cache_dir='/models')

    def process_asr(self, audio_file_path: str) -> AsrResult:
        speech_array, sampling_rate = librosa.load(audio_file_path, sr=self._sampling_rate)
        inputs = self._processor([speech_array], sampling_rate=sampling_rate, return_tensors="pt",
                                 padding=True)
        with torch.no_grad():
            logits = self._model(inputs.input_values, attention_mask=inputs.attention_mask).logits
        predicted_ids = torch.argmax(logits, dim=-1)
        predicted_sentences = self._processor.batch_decode(predicted_ids)
        transcription = predicted_sentences[0]
        return AsrResult(words=transcription.split(' '), full_text=transcription,
                         words_time_alignment=None)


if __name__ == '__main__':
    SpeechbrainAsrProcessor(
        os.environ['MODEL_NAME'],
        int(os.environ['SAMPLING_RATE'])
    ).start_processor()
